<?php

/*
  --------------------------------------------------------------------------
  Integer Base Interconverter

  This program takes an argument in any given base from 2 to 36 and computes
  its equivalent in any other base in the same range.

  Author:
  Jay Tanner - 2019

  Current Web Site
  www.NeoProgrammics.com

  Development Language:
  PHP v7.1.8

  License:
  This program source code and any related data and graphics are released
  under the provisions of the Creative Commons Attribution-ShareAlike 4.0
  International license.

  https://creativecommons.org/licenses/by-sa/4.0/legalcode
  --------------------------------------------------------------------------

   Since this version only works with integers, rather than generate
   errors, any decimal points will be removed as if they never existed.

*/


// ---------------------------------------------
// Initialize

   $ProgramVersion = "v2.7"; // INTEGER ONLY VERSION

// -------------------------------------------------------------
// Set the path to the program to be executed when the submit
// button is clicked.  In this case, it is this program itself.
// It modifies itself each time it is called.

   $RunThisProgram = $_SERVER["SCRIPT_NAME"];

// -----------------
// Get current year.

   $cYear = gmdate("Y");

// Generate random random from/to bases demonstrating
// precision breakdown problem.
   $rFromBase = $rToBase = 0;

    while($rFromBase == $rToBase)
         {
          $rFromBase = mt_rand(8, 36);
          $rToBase = mt_rand(8, 36);
         }

// Generate 50-digit random number for example.
   $Xb = Ltrim(Random_Digits_Base_B (40, $rFromBase), "0");

// We first convert Xb to X10
   $X10 = bcBaseB_Int_To_Base10($Xb, $rFromBase);

// We now convert X10 to ToBase output
   $Y = trim(bcBase10_Int_To_BaseB($X10, $rToBase));

   $Yphp = strtoupper(base_convert($Xb, $rFromBase, $rToBase));

// Determine if Native PHP is correct or in error.
   $YStatus = ($Yphp == $Y)? "OK":"NOT EXACT";

   $ExampOut = "<pre>$Xb base $rFromBase\n=\n$Y base $rToBase\nTRUE EXACT VALUE\n\n$Yphp\nNative PHP function value = $YStatus\n\nCompare:\n$Y = TRUE EXACT VALUE\n$Yphp = $YStatus</pre>";

// -------------------------------------------------------------
// Read input arguments from interface.

   $arg1 = strtoupper(trim(@$_POST["arg1"]));
   if ($arg1 == '')   {$arg1 = Random_Digits_Base_B (mt_rand(7,50), $rFromBase);}

   $arg2 = strtoupper(trim(@$_POST["arg2"]));
   if ($arg2 == ''){$arg2 = $rFromBase;}

   $arg3 = strtoupper(trim(@$_POST["arg3"]));
   if ($arg3 == ''){$arg3 = $rToBase;}

// Remove any decimal point, positive signs and spaces.
// They will be treated as if they never existed.
   $arg1 = str_replace(".", '', $arg1);
   $arg1 = str_replace("+", '', $arg1);
   $arg1 = str_replace(" ", '', $arg1);

   $TimerStarted = $DigitCounts = '';

// -----------------------------------------------------------
// Check for invalid argument.  In this case, must be numeric.
// On error, null output (nothing printed).

   if (1 != 1) {$out = '';}

 else

// -------------------------------------------------------------
// If no error, call the custom function
// using the input argument.

{

// Compute base 10 equivalent of BaseB input value.
// arg1 = x
// arg2 = From base
// arg3 = To base


// Test if arguments are valid.


   $BaseDigits = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $arg2);

       $ErrorMssg = (bcX_Is_Base_B($arg1, $arg2) === TRUE)?
       '' : "<div style='line-height:175%; text-align:center;'><b style='color:red; font-size:15px;'>ERROR<br><br>Invalid X value for base $arg2.&nbsp; At least one invalid input digit was detected.<br>Numbers expressed in base $arg2 can only use the following $arg2 digits:<br>$BaseDigits</b></div>";


//     Error if invalid from/to base
       if ($arg2 < 2 or $arg2 > 36 or $arg3 < 2 or $arg3 > 36)
       {
       $ErrorMssg = "<b style='color:red; font-size:15px;'>ERROR<br><br>Invalid base value.&nbsp; The base values must both be in the range from 2 to 36.</b>";
       }

if ($ErrorMssg != '')
   {
    $out = $ErrorMssg;
    $DigitCounts = $ErrorMssg;
    $TimerStopped = 0;
    $out = '';
   }

else
   {
   $TimerStarted = MicroTime(TRUE);
   $b10 = bcBaseB_Int_To_Base10 ($arg1, $arg2);
   $bbOut  = bcBase10_Int_To_BaseB ($b10,  $arg3);

   $TimerStopped = sprintf("%4.3f", MicroTime(TRUE) - $TimerStarted);
   $TimerStopped = Seconds_To_HMS($TimerStopped, 3);

// Count digits in input and output values.
   $bbInpDigits = strlen($arg1);
   $bbOutDigits = strlen($bbOut);

   $DigitCounts = "<span style='font-size:14px;'>The <b>$bbInpDigits</b>-digit base <b>$arg2</b> integer above EXACTLY equates to the following <b>$bbOutDigits</b>-digit base <b>$arg3</b> integer below:</span>";

   $out = "$bbOut";
   }
}

// Set number of text lines for output text area.
   $TextLines = 2 + floor(strlen($out) / 80);

// --------------------------
// Generate HTML page output.

print <<< _HTML

<!DOCTYPE HTML>

<html>

<head>

<title>Integer Base Interconverter</title>

<meta http-equiv="content-type" content="text/html; charset=ISO-8859-1">

<!-- Set search engine permissions -->
<meta name="robots"    content="noindex,nofollow">
<meta name="googlebot" content="noindex,nofollow">

<style type='text/css'>
BODY
{color:silver; background-color:black;  font-family:Verdana; font-size:15px}

TABLE
{font-size:12px; border: 1px solid black;}

TD
{color:black; background-color:white;  line-height:125%; font-size:12px; padding:6px; text-align:center;}

DIV
{color:black; background-color:white;  font-family:Verdana; text-align:justify; padding:4px; font-size:12px;}

PRE
{color:black; background-color:white;  font-family:Consolas,monospace; font-size:15px; font-weight:bold; text-align:left; padding:4px;}

CODE
{font-family:Consolas,monospace; font-size:15px; font-weight:normal;}

TEXTAREA
{color:black; background-color:white; font-family:Consolas,monospace; font-size:20px; font-weight:bold; box-shadow:3px 3px 3px #888888; border-radius: 8px; border:1px solid black; padding:4px;}

INPUT[type="text"]::-ms-clear {width:0; height:0;}

INPUT[type="text"]
{color:black; background-color:white; font-size:20px; font-family:monospace; font-weight:bold; text-align:center; box-shadow:2px 2px 3px #666666; border:2px solid black;}

INPUT[type="text"]:focus
{background-color:white; box-shadow:2px 2px 3px #888888; font-family:monospace; font-size:20px; border:2px solid red; text-align:center; font-weight:bold;}

INPUT[type="submit"]
{font-size:16px; font-weight:bold;}

INPUT[type="submit"]:hover
{box-shadow:2px 2px 5px #222222; font-size:16px; font-weight:bold;}

INPUT[type="checkbox"]:checked
{box-shadow:2px 2px 3px #222222;}

HR {background-color:black; height:2px; border:0px;}

A:link,  A:visited {background-color:transparent; color:blue; font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}
A:hover, A:active  {background-color:transparent; color:red;  font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */
</style>

</head>



<body style='color:white; background-color:black; font-family:Verdana; font-size:12px;'>

<table width="666" align="bottom" bgcolor='black'>
<tr><td style='color:white; background-color:#0000CC; font-size:15px; text-align:center; font-weight:bold; border:2px solid white; border-radius:8px 8px 0px 0px; padding:4px;'>
Integer Base (Radix) Interconverter
</td></tr>

<tr><td style='border-radius:0px 0px 8px 8px;'>
<form  name="form1" method="post" action="$RunThisProgram">
<span style='font-size:14px;'>
<b>This program converts any integer (X) value from one base (radix) system into another.</b>
<br><br>
The initial default startup values are randomized.&nbsp; Simply replace them with your own working values.</span><br><br>
<div style='text-align:center;'><b>Enter Integer X</b>&nbsp; <span style='font-size:12px;'>(Expressed in any base from <b>2</b> to <b>36</b>)</span></div>

<textarea name="arg1" cols='80' rows='3'>
$arg1
</textarea>

<br><br>

<div style='font-size:14px; text-align:center; background-color:#E0E0E0; border:1px solid black; padding:8px; border-radius:8px;'>
<b><input name="button1" type="submit" value="Convert" OnClick="TextOut.innerHTML='Working ...'; DivOut.innerHTML='';">&nbsp; Integer X&nbsp;value&nbsp;above&nbsp; from base <input name="arg2"  type="text"   value="$arg2" size='3'  maxlength='2'>&nbsp;&nbsp;&nbsp;to base</b>&nbsp;<input name="arg3"  type="text"   value="$arg3" size='3' maxlength='2'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span style='font-size:14px;'>(Where: <b>base</b> = <b>2</b> to <b>36</b>)</span>
</div>
</form>

<br>


<div ID="DivOut" style='text-align:center;'>$DigitCounts<br></div>
<br>


<textarea ID="TextOut" rows="$TextLines" cols='80' readonly='readonly' style='background-color:LightYellow;'>
$out
</textarea>

<br><br><br>



<div style='font-size:10px; text-align:center;'>PHP Math Tool by Jay Tanner - $cYear</div>

</td></tr>




</table>


</body>
</html>

_HTML;


// ****************************************************************************

/*
   ========================================================================
   This function will convert an arbitrary precision positive base 10
   X value into its equivalent in any base from 2 to 36.  The X value
   is NOT restricted to integers and also allows fractional values.

   ARGUMENTS:
   $XBase10 = Base 10 numeric string to be to converted
              into its BaseB equivalent.

   $BaseB = Base (2 to 36) in which to interpret input digit string.

   ERRORS:
   On error FALSE is returned.  An error occurs if either
   of the arguments is non-numeric or out of valid range.

   This is a level 2 function.
   ========================================================================
*/

   function bcX_Base_10_To_Base_B ($xBase10, $BaseB, $TruncationLimit=0)
{
   $p = 2000; // Decimal precision control factor (Don't ask).

   $q =  $TruncationLimit; // Decimal truncation limit.

   $x = trim($xBase10);  $sign = (substr($x,0,1) != "-")? "" : "-";
   $x = str_replace("-","", $x);

   if (strpos($x, ".") === FALSE){$x .= ".0";}

   list($i, $f) = preg_split("[\.]", $x);  $f = "0.$f";

   $IntPartOut = bcXBase_10_Int_To_Base_B ($i, $BaseB);

   $w = bcmul(bcPow($BaseB, $p), $f);

   $fp = bcXBase_10_Int_To_Base_B ($w, $BaseB);

   $zeros = str_repeat("0", $p - strlen($fp));

   if ($q > 0) {$fp = substr($fp, 0, $q-0);}

   $FracPartOut = ".$zeros$fp";

   $FracPartOut = rtrim(rtrim($FracPartOut,'0'), '.');

   return "$sign$IntPartOut$FracPartOut";
}



/*
   ========================================================================
   This boolean function tests to see of the given argument is a valid
   base B number.

   A digit spectrum for the given base is created.  Then, if any of the
   digits given in the argument are not found within the digit spectrum,
   FALSE is returned to indicate the error.

   If no error found, then TRUE is returned.
   ========================================================================
*/

   function bcX_Is_Base_B ($NumArgStr, $BaseB)
{

// Read numeric string and remove any decimal or numerical sign.
   $X = str_replace(".", '', strtoupper(trim($NumArgStr)));
   $X = str_replace("-", '', $X);
   $X = str_replace("+", '', $X);

   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $BaseB);

// Cycle through every digit.  Error if any digit is not found
// in DigitsSpectrum string and FALSE is returned.  Otherwise,
// TRUE is returned.
   $DigitsCount = strlen($X);
   for ($i=0;   $i < $DigitsCount;   $i++)
       {
        $digit = substr($X, $i, 1);
        if (strpos($DigitsSpectrum, $digit) === FALSE) {return FALSE;}
       }
   return TRUE;
}




/*
   ========================================================================
   This function will convert an arbitrary precision integer
   in any base from 2 to 36 into its equivalent in base 10.

   ARGUMENTS:
   $Xb = Base B integer string to be converted
         into its base 10 equivalent.

   $FromBase = Base of input digit string (2 to 36).

   ERRORS:
   On error FALSE is returned.  An error occurs if either
   of the arguments is invalid.

   ========================================================================
*/


   function bcBaseB_Int_To_Base10 ($XbStr, $FromBase)
{

// Read input arguments.
   $Xb = strtoupper(trim($XbStr));
   $b  = trim($FromBase);

// ERROR if base of outside of range 2 to 36.
   if ($b < 2 or $b > 36) {return FALSE;}

// Remember any numerical sign and work with absolute value.
// The numerical sign will be restored at the end.
   $sign = (substr($Xb, 0,1) == '-')? '-' :'';
   if ($sign == '-') {$Xb = substr($Xb, 1, strlen($Xb));}

// Count digits in base b integer X.
   $XbDigitsCount = strlen($Xb);

// Initialize base 10 summation accumulator
// and initial power (= 1) of the base.
   $X10 = 0;
   $PowB = 1;

// Define the valid digits spectrum for the given base.
   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $b);

// Perform loop to compute the base 10
// equivalent of the base b integer Xb.
   for ($i=0;   $i < $XbDigitsCount;   $i++)
       {
//      Get current base b digit from Xb.
        $db = substr($Xb, $XbDigitsCount - $i - 1, 1);

//      Get base 10 numerical value (= string index) of digit.
        $d10 = strpos($DigitsSpectrum, $db);

//      If FALSE is returned, it means that the base b digit was
//      not found within the valid base b DigitsSpectrum string.
        if ($d10 === FALSE) {return FALSE;}

//      Compute and accumulate value of current base 10 term.
        $X10 = bcadd($X10, bcmul($PowB, $d10));

//      Update power of base for next cycle.
        $PowB = bcmul($PowB, $b);
       }

   return $sign . $X10;

} // End of  bcBaseB_Int_To_Base10(...)



/*
   ========================================================================
   This function will convert an arbitrary precision positive decimal
   integer into its equivalent in any given base from 2 to 36.

   ARGUMENTS:
   $Base10IntStr = Base-10 integer string to be to converted
                   into its BaseB equivalent.

   $BaseB = Base of converted output (2 to 36).

   ERRORS:
   On error FALSE is returned.  An error occurs if either
   of the arguments is non-numeric or out of valid range.
   ========================================================================
*/

   function bcBase10_Int_To_BaseB ($x10IntStr, $BaseB)
{
// ---------------------------------------
// Error if either argument is non-numeric
// or if base of out of range (2 to 36).

   if (!is_numeric($x10IntStr) or !is_numeric($BaseB)) {return FALSE;}

   if ($BaseB < 2 or $BaseB > 36) {return FALSE;}

// ------------------------------------------
// Define available digits for bases 2 to 36.

   $BaseDigits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

   $w = '';
   $x = trim($x10IntStr);

// Remember any numerical sign and work with absolute value.
// The numerical sign will be restored at the end.
   $sign = (substr($x, 0,1) == '-')? '-' :'';
   if ($sign == '-') {$x = substr($x, 1, strlen($x));}

   if (bccomp($x, "0") == 0) {return "0";}

   while (bccomp($x, "0") > 0)
         {
          $w = substr($BaseDigits, bcmod($x, $BaseB), 1) . $w;
          $x = bcdiv($x, $BaseB);
         }

   return $sign . $w;

}  // End of  bcBase10_Int_To_BaseB(...)


/*
   ========================================================================
   Quick utility function to convert seconds of arc or time to
   HMS resolved to up to 3 decimals.   The default setting is
   to the nearest second.

   No symbols are attached to the returned HMS values.
   The value of the hours may exceed 24h

   No error checking is done.
   ========================================================================
*/

   function Seconds_To_HMS ($SecondsArg, $NumDecimals=0, $PosSign='')
{

// -----------------------------------------------
// Construct output format string for decimal part
// based on the (NumDecimals) argument.

   $ssOutputFormat = "%2.$NumDecimals" . "f";

// --------------------------------------------------------------
// Read output numerical sign symbol.  A positive value will have
// a plus (+) symbol attached if this value is '+', otherwise it
// will NOT have any plus symbol.  Negative values will not be
// effected by this setting.

   $SignOutput = trim($PosSign);

// --------------------------------------------
// Remember numerical sign of seconds argument.

   $sign = ($SecondsArg < 0)? '-' : '+';

   if ($SignOutput == '') {$sign = $SignOutput;}

// ------------------------------------------
// Compute HMS elements equivalent to seconds
// and rounded-off to the nearest specified
// number of decimals (0 to 3).

   $DegHrs  = abs($SecondsArg) / 3600;
   $dh      = floor($DegHrs);
   $minutes = 60*($DegHrs - $dh);
   $mm      = floor($minutes);
   $seconds = 60*($minutes - $mm);

   $ss = ($NumDecimals == 0)? floor($seconds + 0.5) : $seconds;

// ------------------------------------
// Patch for that blasted '60s problem.

   if ($ss == 60) {$ss=0;  $mm++;}
   if ($mm == 60) {$mm=0;  $hh++;}

// ----------------------------------
// Format the DHMS values for output.

   $dh = sprintf("%02d", $dh);
   $mm = sprintf("%02d", $mm);
   $ss = sprintf($ssOutputFormat, $ss);

// ----------------------------------------------------------------
// Put ss value of integer part into double-digit format if ss < 10
// The value 9.123 would be converted into 09.123

   $ss = ($ss < 10)? "0" . trim($ss) : trim($ss);

// ---------------------------------
// Construct and return the angle or
// time elements string.

   return "$sign$dh" . ":$mm" . ":$ss";

} // End of  Seconds_To_HMS(...)


/*
   ========================================================================
   This function generates a string of any number of
   random digits in any given base from 2 to 36.

   ERRORS: No error checking is done.
   ========================================================================
*/

   function Random_Digits_Base_B ($NumDigits=1, $base=10)
{

// ---------------
// Read arguments.

   $n = trim($NumDigits);
   $b = trim($base);

// -------------------------------------------
// Define available digits for the given base.

   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $b);

// ----------------------------------------
// Define accumulated digits output string.

   $RandDigits = '';

// -------------------------------------------
// Construct an n-digit random sequence in the
// given base, one digit at a time using a
// random 50/50 digit alternation method.
// There must be at least one digit.

   for ($i=0;   $i < $n;   $i++)
  {
   if (MT_Rand() % 2 == 0)

      {$RandDigits .= substr($DigitsSpectrum, MT_Rand(0, $b-1), 1);}
   else
      {$RandDigits  = substr($DigitsSpectrum, MT_Rand(0, $b-1), 1) . $RandDigits;}
  }

   return "$RandDigits";

} // End of  Random_Digits_Base_B(...)



?>


